
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'cherryrgb_cli' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'cherryrgb_cli'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'cherryrgb_cli' {
            [CompletionResult]::new('-p', 'p', [CompletionResultType]::ParameterName, 'Must be specified if multiple cherry products are detected. Interpreted as hex, if prefixed with ''0x'', as dec otherwise')
            [CompletionResult]::new('--product-id', 'product-id', [CompletionResultType]::ParameterName, 'Must be specified if multiple cherry products are detected. Interpreted as hex, if prefixed with ''0x'', as dec otherwise')
            [CompletionResult]::new('-b', 'b', [CompletionResultType]::ParameterName, 'Set brightness')
            [CompletionResult]::new('--brightness', 'brightness', [CompletionResultType]::ParameterName, 'Set brightness')
            [CompletionResult]::new('-d', 'd', [CompletionResultType]::ParameterName, 'Enable debug output')
            [CompletionResult]::new('--debug', 'debug', [CompletionResultType]::ParameterName, 'Enable debug output')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('-V', 'V', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', 'version', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('animation', 'animation', [CompletionResultType]::ParameterValue, 'Configure RGB keyboard illumination')
            [CompletionResult]::new('custom-colors', 'custom-colors', [CompletionResultType]::ParameterValue, 'Configure custom RGB colors')
            [CompletionResult]::new('color-profile-file', 'color-profile-file', [CompletionResultType]::ParameterValue, 'Configure custom RGB colors from file')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'cherryrgb_cli;animation' {
            [CompletionResult]::new('-r', 'r', [CompletionResultType]::ParameterName, 'Enable rainbow colors')
            [CompletionResult]::new('--rainbow', 'rainbow', [CompletionResultType]::ParameterName, 'Enable rainbow colors')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'cherryrgb_cli;custom-colors' {
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'cherryrgb_cli;color-profile-file' {
            [CompletionResult]::new('-k', 'k', [CompletionResultType]::ParameterName, 'If enabled, modifies existing color profile')
            [CompletionResult]::new('--keep-existing-colors', 'keep-existing-colors', [CompletionResultType]::ParameterName, 'If enabled, modifies existing color profile')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'cherryrgb_cli;help' {
            [CompletionResult]::new('animation', 'animation', [CompletionResultType]::ParameterValue, 'Configure RGB keyboard illumination')
            [CompletionResult]::new('custom-colors', 'custom-colors', [CompletionResultType]::ParameterValue, 'Configure custom RGB colors')
            [CompletionResult]::new('color-profile-file', 'color-profile-file', [CompletionResultType]::ParameterValue, 'Configure custom RGB colors from file')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'cherryrgb_cli;help;animation' {
            break
        }
        'cherryrgb_cli;help;custom-colors' {
            break
        }
        'cherryrgb_cli;help;color-profile-file' {
            break
        }
        'cherryrgb_cli;help;help' {
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
