
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'cherryrgb_service' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'cherryrgb_service'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'cherryrgb_service' {
            [CompletionResult]::new('-p', 'p', [CompletionResultType]::ParameterName, 'Must be specified if multiple cherry products are detected')
            [CompletionResult]::new('--product-id', 'product-id', [CompletionResultType]::ParameterName, 'Must be specified if multiple cherry products are detected')
            [CompletionResult]::new('-s', 's', [CompletionResultType]::ParameterName, 'Path of listening socket to create')
            [CompletionResult]::new('--socket', 'socket', [CompletionResultType]::ParameterName, 'Path of listening socket to create')
            [CompletionResult]::new('-m', 'm', [CompletionResultType]::ParameterName, 'Permissions of the socket (octal)')
            [CompletionResult]::new('--socketmode', 'socketmode', [CompletionResultType]::ParameterName, 'Permissions of the socket (octal)')
            [CompletionResult]::new('-g', 'g', [CompletionResultType]::ParameterName, 'Group of the socket')
            [CompletionResult]::new('--socketgroup', 'socketgroup', [CompletionResultType]::ParameterName, 'Group of the socket')
            [CompletionResult]::new('-d', 'd', [CompletionResultType]::ParameterName, 'Enable debug output')
            [CompletionResult]::new('--debug', 'debug', [CompletionResultType]::ParameterName, 'Enable debug output')
            [CompletionResult]::new('-h', 'h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', 'help', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('-V', 'V', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', 'version', [CompletionResultType]::ParameterName, 'Print version')
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
